@@ -1,14 +1,14 @@
-# $Id: TCP.pm 1976 2006-06-06 03:07:35Z immute $
+# $Id: TCP.pm 2357 2008-06-20 17:41:54Z rcaputo $
 
 package POE::Component::Server::TCP;
 
 use strict;
 
 use vars qw($VERSION);
-$VERSION = do {my($r)=(q$Revision: 1976 $=~/(\d+)/);sprintf"1.%04d",$r};
+$VERSION = do {my($r)=(q$Revision: 2357 $=~/(\d+)/);sprintf"1.%04d",$r};
 
 use Carp qw(carp croak);
-use Socket qw(INADDR_ANY inet_ntoa inet_aton AF_UNIX PF_UNIX);
+use Socket qw(INADDR_ANY inet_ntoa inet_aton AF_INET AF_UNIX PF_UNIX);
 use Errno qw(ECONNABORTED ECONNRESET);
 
 # Explicit use to import the parameter constants.
@@ -44,7 +44,7 @@ sub new {
   my $address = delete $param{Address};
   my $hname   = delete $param{Hostname};
   my $port    = delete $param{Port};
-  my $domain  = delete $param{Domain};
+  my $domain  = delete($param{Domain}) || AF_INET;
   my $concurrency = delete $param{Concurrency};
 
   foreach (
@@ -63,10 +63,12 @@ sub new {
   my $high_event      = delete $param{ClientHigh};
   my $low_event       = delete $param{ClientLow};
 
-  # this is ugly, but now its elegant :)  grep++
-  my $foo = grep { defined $_ } ($high_mark_level, $low_mark_level, $high_event, $low_event);
-  if ($foo > 0) {
-    croak "If you use the Mark settings, you must define all four" unless $foo == 4;
+  my $mark_param_count = (
+    grep { defined $_ }
+    ($high_mark_level, $low_mark_level, $high_event, $low_event)
+  );
+  if ($mark_param_count and $mark_param_count < 4) {
+    croak "If you use the Mark settings, you must define all four";
   }
 
   $high_event = sub { } unless defined $high_event;
@@ -103,7 +105,7 @@ sub new {
   # Defaults.
 
   $concurrency = -1 unless defined $concurrency;
-  my $accept_session;
+  my $accept_session_id;
 
   if (!defined $address && defined $hname) {
     $address = inet_aton($hname);
@@ -181,8 +183,7 @@ sub new {
               # to pull most of this into a base class and derive
               # TCP and UNIX versions from that.
               if (
-                defined $domain and
-                ($domain == AF_UNIX or $domain == PF_UNIX)
+                $domain == AF_UNIX or $domain == PF_UNIX
               ) {
                 $heap->{remote_ip} = "LOCAL";
               }
@@ -199,18 +200,25 @@ sub new {
               $heap->{client} = POE::Wheel::ReadWrite->new(
                 Handle       => splice(@_, ARG0, 1),
                 Driver       => POE::Driver::SysRW->new(),
-                _get_filters($client_filter, $client_infilter, $client_outfilter), 
+                _get_filters(
+                  $client_filter,
+                  $client_infilter,
+                  $client_outfilter
+                ),
                 InputEvent   => 'tcp_server_got_input',
                 ErrorEvent   => 'tcp_server_got_error',
                 FlushedEvent => 'tcp_server_got_flush',
-                do {
-                  $foo ? return (
-                    HighMark => $high_mark_level,
+
+                (
+                  $mark_param_count
+                  ? (
+                    HighMark  => $high_mark_level,
                     HighEvent => 'tcp_server_got_high',
-                    LowMark => $low_mark_level,
-                    LowEvent => 'tcp_server_got_low',
-                  ) : ();
-                },
+                    LowMark   => $low_mark_level,
+                    LowEvent  => 'tcp_server_got_low',
+                  )
+                  : ()
+                ),
               );
 
               $client_connected->(@_);
@@ -266,10 +274,10 @@ sub new {
             _stop => sub {
               ## concurrency on close
               DEBUG and warn(
-                "$$: $alias _stop accept_session = $accept_session"
+                "$$: $alias _stop accept_session = $accept_session_id"
               );
-              if( defined $accept_session ) {
-                $_[KERNEL]->call( $accept_session, 'disconnected' );
+              if( defined $accept_session_id ) {
+                $_[KERNEL]->call( $accept_session_id, 'disconnected' );
               }
               else {
                 # This means that the Server::TCP was shutdown before
@@ -322,10 +330,11 @@ sub new {
     $orig_accept_callback->(@_);
   };
 
-  # Create the session, at long last.  This is done inline so that
-  # closures can customize it.
+  # Create the session, at long last.
+  # This is done inline so that closures can customize it.
+  # We save the accept session's ID to avoid self reference.
 
-  $accept_session = $session_type->create(
+  $accept_session_id = $session_type->create(
     @$session_params,
     inline_states => {
       _start => sub {
@@ -363,6 +372,13 @@ sub new {
           "$$: $_[HEAP]->{alias} Connection closed ",
           "($_[HEAP]->{connections} open)"
         );
+        if ($_[HEAP]->{connections} < 0) {
+          warn(
+            "Excessive 'disconnected' event ",
+            "from $_[CALLER_FILE] at line $_[CALLER_LINE]\n"
+          );
+          $_[HEAP]->{connections} = 0;
+        }
         if( $_[HEAP]->{concurrency} != -1 and $_[HEAP]->{listener} ) {
           if( $_[HEAP]->{connections} == ($_[HEAP]->{concurrency}-1) ) {
             DEBUG and warn(
@@ -408,21 +424,17 @@ sub new {
       # Dummy states to prevent warnings.
       _stop   => sub {
         DEBUG and warn "$$: $_[HEAP]->{alias} _stop";
-        undef($accept_session);
+        undef($accept_session_id);
         return 0;
       },
       _child  => sub { },
     },
 
     args => $args,
-  );
+  )->ID;
 
-  $accept_session = $accept_session->ID;
-
-  # Return undef so nobody can use the POE::Session reference.  This
-  # isn't very friendly, but it saves grief later.
-  # Maybe we can at least return the session ID?
-  undef;
+  # Return the session ID.
+  return $accept_session_id;
 }
 
 sub _get_filters {
@@ -433,7 +445,9 @@ sub _get_filters {
         "OutputFilter" => _load_filter($client_outfilter)
       );
       if (defined $client_filter) {
-        carp "Using ClientFilter with ClientInputFilter or ClientOutputFilter is a noop";
+        carp(
+          "ClientFilter ignored with ClientInputFilter or ClientOutputFilter"
+        );
       }
     }
     elsif (defined $client_filter) {
@@ -444,6 +458,7 @@ sub _get_filters {
     }
 
 }
+
 # Get something: either arrayref, ref, or string
 # Return filter
 sub _load_filter {
@@ -469,7 +484,7 @@ sub _load_filter {
     }
 }
 
-# Test if a Filter can be loaded, return sucess or failure
+# Test if a Filter can be loaded, return success or failure
 sub _test_filter {
     my $filter = shift;
     my $eval = eval {
@@ -521,11 +536,39 @@ POE::Component::Server::TCP - a simplified TCP server
 
 =head1 SYNOPSIS
 
+  #!perl
+
+  use warnings;
+  use strict;
+
+  use POE qw(Component::Server::TCP);
+
+  POE::Component::Server::TCP->new(
+    Port => 12345,
+    ClientConnected => sub {
+      print "got a connection from $_[HEAP]{remote_ip}\n";
+      $_[HEAP]{client}->put("Smile from the server.");
+    },
+    ClientInput => sub {
+      my $client_input = $_[ARG0];
+      $client_input =~ tr[a-zA-Z][n-ma-zN-MA-Z];
+      $_[HEAP]{client}->put($client_input);
+    },
+    ClientDisconnected => sub {
+      print "client from $_[HEAP]{remote_ip} disconnected\n";
+    },
+  );
+
+  POE::Kernel->run;
+  exit;
+
+-><- OLD SYNOPSIS FOLLOWS
+
   use POE qw(Component::Server::TCP);
 
   ### First form just accepts connections.
 
-  POE::Component::Server::TCP->new(
+  my $acceptor_session_id = POE::Component::Server::TCP->new(
     Port     => $bind_port,
     Address  => $bind_address,    # Optional.
     Hostname => $bind_hostname,   # Optional.
@@ -537,7 +580,7 @@ POE::Component::Server::TCP - a simplified TCP server
 
   ### Second form also handles connections.
 
-  POE::Component::Server::TCP->new(
+  my $acceptor_session_id = POE::Component::Server::TCP->new(
     Port     => $bind_port,
     Address  => $bind_address,      # Optional.
     Hostname => $bind_hostname,     # Optional.
@@ -555,9 +598,9 @@ POE::Component::Server::TCP - a simplified TCP server
     ClientDisconnected => \&handle_client_disconnect, # Optional.
     ClientError        => \&handle_client_error,      # Optional.
     ClientFlushed      => \&handle_client_flush,      # Optional.
-    ClientFilter       => "POE::Filter::Xyz",         # Optional.
-    ClientInputFilter  => "POE::Filter::Xyz",         # Optional.
-    ClientOutputFilter => "POE::Filter::Xyz",         # Optional.
+    ClientFilter       => POE::Filter::Xyz->new()",   # Optional.
+    ClientInputFilter  => POE::Filter::Xyz->new(),    # Optional.
+    ClientOutputFilter => POE::Filter::Xyz->new(),    # Optional.
     ClientShutdownOnError => 0,                       # Optional.
 
     # Optionally define other states for the client session.
@@ -634,13 +677,19 @@ difficult.  A tutorial at http://poe.perl.org/ describes how.
 
 =head1 CONSTRUCTOR PARAMETERS
 
-The new() method can accept quite a lot of parameters.  It always
-returns undef, however.  One must use callbacks to check for errors
-rather than the return value of new().
+=over
+
+=item new
+
+The new() method can accept quite a lot of parameters.  It will return
+the session ID of the acceptor session.  One must use callbacks to
+check for errors rather than the return value of new().
 
 POE::Component::Server::TCP supplies common defaults for most
 callbacks and handlers.
 
+=back
+
 =over 2
 
 =item Acceptor => CODEREF
@@ -687,16 +736,16 @@ Later on, the 'chargen' service can be shut down with:
 
 =item Args => ARRAYREF
 
-Args passes the contents of a ARRAYREF to the ClientConnected callback
-via @_[ARG0..$#_].  It allows you to send extra information to the
-sessions that handle each client connection.
+Args passes the ARRAYREF to the ClientConnected callback as ARG0.
+It allows you to send extra information to the sessions that handle 
+each client connection.
 
 =item ClientConnected => CODEREF
 
 ClientConnected sets the callback used to notify each new client
 session that it is started.  ClientConnected callbacks receive the
 usual POE parameters, plus a copy of whatever was specified in the
-component's C<Args> constructor parameter.
+component's C<Args> constructor parameter, as ARG0.
 
 The ClientConnected callback will not be called within the same
 session context twice.
@@ -732,12 +781,13 @@ shutdown, not the error, that invokes ClientDisconnected callbacks.
 
 =item ClientFilter => ARRAYREF
 
+=item ClientFilter => OBJECT
+
 ClientFilter specifies the type of filter that will parse input from
 each client, and optionally any constructor arguments used to create
 each filter instance.
 
-It takes a POE::Filter class name rather than an object because the
-component must create a new instance for each connection.
+It can take a class name, an arrayref, or a POE::Filter object.
 
 If ClientFilter contains a SCALAR, it defines the name of the
 POE::Filter class.  Default constructor parameters will be used to
@@ -752,6 +802,11 @@ line separator to a vertical bar:
 
   ClientFilter => [ "POE::Filter::Line", Literal => "|" ],
 
+If ClientFilter contains an object, it will have a clone method called
+on it each time a client connects.
+
+  ClientFilter => POE::Filter::Block->new()
+
 ClientFilter is optional.  The component will use "POE::Filter::Line"
 if it is omitted.
 
@@ -762,10 +817,14 @@ that filter class.
 
 =item ClientInputFilter => ARRAYREF
 
+=item ClientInputFilter => OBJECT
+
 =item ClientOutputFilter => SCALAR
 
 =item ClientOutputFilter => ARRAYREF
 
+=item ClientOutputFilter => OBJECT
+
 ClientInputFilter and ClientOutputFilter act like ClientFilter, but
 they allow programs to specify different filters for input and output.
 Both must be used together.
@@ -774,6 +833,7 @@ Usage is the same as ClientFilter.
 
   ClientInputFilter  => [ "POE::Filter::Line", Literal => "|" ],
   ClientOutputFilter => "POE::Filter::Stream",
+  ClientInputFilter  => POE::Filter::Block->new(),
 
 =item ClientInput => CODEREF
 
@@ -897,7 +957,7 @@ Started is optional.
 =item Concurrency => SCALAR
 
 Controls the number of connections that may be open at the same time.
-Defaults to -1, which means unlimited number of simutaneous connections.
+Defaults to -1, which means unlimited number of simultaneous connections.
 0 means no connections.  This value may be set via the
 C<set_concurrency> event, see L<EVENTS>.
 
@@ -972,15 +1032,19 @@ This component currently does not accept many of the options that
 POE::Wheel::SocketFactory does.
 
 This component will not bind to several addresses at once.  This may
-be a limitation in SocketFactory.
+be a limitation in SocketFactory, but it's not by design.
 
 This component needs more complex error handling which appends for
 construction errors and replaces for runtime errors, instead of
 replacing for all.
 
+Some use cases require different session classes for the listener and
+the connection handlers.  This isn't currently supported.  Please send
+patches. :)
+
 =head1 AUTHORS & COPYRIGHTS
 
-POE::Component::Server::TCP is Copyright 2000-2001 by Rocco Caputo.
+POE::Component::Server::TCP is Copyright 2000-2006 by Rocco Caputo.
 All rights are reserved.  POE::Component::Server::TCP is free
 software, and it may be redistributed and/or modified under the same
 terms as Perl itself.
@@ -992,3 +1056,6 @@ POE::Component::Server::TCP is based on code, used with permission,
 from Jos Boumans E<lt>kane@cpan.orgE<gt>.
 
 =cut
+
+# rocco // vim: ts=2 sw=2 expandtab
+# TODO - Redocument.
