@@ -1,4 +1,4 @@
-# $Id: Stream.pm 1920 2006-04-02 07:17:33Z rcaputo $
+# $Id: Stream.pm 2337 2008-05-27 07:09:15Z rcaputo $
 
 package POE::Filter::Stream;
 
@@ -6,7 +6,7 @@ use strict;
 use POE::Filter;
 
 use vars qw($VERSION @ISA);
-$VERSION = do {my($r)=(q$Revision: 1920 $=~/(\d+)/);sprintf"1.%04d",$r};
+$VERSION = do {my($r)=(q$Revision: 2337 $=~/(\d+)/);sprintf"1.%04d",$r};
 @ISA = qw(POE::Filter);
 
 #------------------------------------------------------------------------------
@@ -61,40 +61,70 @@ sub get_pending {
   return undef;
 }
 
-###############################################################################
 1;
 
 __END__
 
 =head1 NAME
 
-POE::Filter::Stream - pass through data unchanged (a do-nothing filter)
+POE::Filter::Stream - a no-op filter that passes data through unchanged
 
 =head1 SYNOPSIS
 
-  $filter = POE::Filter::Stream->new();
-  $arrayref_of_logical_chunks =
-    $filter->get($arrayref_of_raw_chunks_from_driver);
-  $arrayref_of_streamable_chunks_for_driver =
-     $filter->put($arrayref_of_logical_chunks);
+  #!perl
+
+  use Term::ReadKey;
+  use POE qw(Wheel::ReadWrite Filter::Stream);
+
+  POE::Session->create(
+    inline_states => {
+      _start => sub {
+        ReadMode "ultra-raw";
+        $_[HEAP]{io} = POE::Wheel::ReadWrite->new(
+          InputHandle => \*STDIN,
+          OutputHandle => \*STDOUT,
+          InputEvent => "got_some_data",
+          Filter => POE::Filter::Stream->new(),
+        );
+      },
+      got_some_data => sub {
+        $_[HEAP]{io}->put("<$_[ARG0]>");
+        delete $_[HEAP]{io} if $_[ARG0] eq "\cC";
+      },
+      _stop => sub {
+        ReadMode "restore";
+        print "\n";
+      },
+    }
+  );
+
+  POE::Kernel->run();
+  exit;
 
 =head1 DESCRIPTION
 
-This filter passes data through unchanged.
+POE::Filter::Stream passes data through without changing it.  It
+follows POE::Filter's API and implements no new functionality.
+
+In the L</SYNOPSIS>, POE::Filter::Stream is used to collect keystrokes
+without any interpretation and display output without any
+embellishments.
 
 =head1 SEE ALSO
 
-POE::Filter.
+L<POE::Filter> for more information about filters in general.
 
 The SEE ALSO section in L<POE> contains a table of contents covering
 the entire POE distribution.
 
 =head1 BUGS
 
-Oh, probably some.
+None known.
 
 =head1 AUTHORS & COPYRIGHTS
 
 Please see L<POE> for more information about authors and contributors.
 
 =cut
+
+# rocco // vim: ts=2 sw=2 expandtab
