@@ -1,4 +1,4 @@
-# $Id: TkCommon.pm 2163 2007-01-03 06:13:16Z rcaputo $
+# $Id: TkCommon.pm 2329 2008-05-25 23:01:58Z rcaputo $
 
 # The common bits of our system-specific Tk event loops.  This is
 # everything but file handling.
@@ -10,7 +10,7 @@ package POE::Loop::TkCommon;
 use POE::Loop::PerlSignals;
 
 use vars qw($VERSION);
-$VERSION = do {my($r)=(q$Revision: 2163 $=~/(\d+)/);sprintf"1.%04d",$r};
+$VERSION = do {my($r)=(q$Revision: 2329 $=~/(\d+)/);sprintf"1.%04d",$r};
 
 use Tk 800.021;
 use 5.00503;
@@ -23,7 +23,6 @@ use strict;
 use Tk qw(DoOneEvent DONT_WAIT ALL_EVENTS);
 
 my $_watcher_time;
-my $_waiting_for_event = 0;
 
 #------------------------------------------------------------------------------
 # Signal handler maintenance functions.
@@ -34,15 +33,11 @@ sub loop_attach_uidestroy {
   $window->OnDestroy(
     sub {
       if ($self->_data_ses_count()) {
-
         $self->_dispatch_event(
           $self, $self,
           EN_SIGNAL, ET_SIGNAL, [ 'UIDESTROY' ],
           __FILE__, __LINE__, time(), -__LINE__
         );
-
-        # Flag the main loop as done.
-        $_waiting_for_event = 0;
       }
     }
   );
@@ -56,10 +51,9 @@ sub loop_resume_time_watcher {
   $self->loop_pause_time_watcher();
   my $timeout = $next_time - time();
 
-  return if $timeout < 0;
-  $_waiting_for_event = 1;
+  $timeout = "idle" if $timeout < 0;
   $_watcher_time = $poe_main_window->after(
-    $timeout * 1000, [ sub { $_waiting_for_event = 0 } ]
+    $timeout * 1000, [ sub { } ]
   );
 }
 
@@ -74,7 +68,6 @@ sub loop_pause_time_watcher {
     $_watcher_time->cancel() if $_watcher_time->can("cancel");
     $_watcher_time = undef;
   }
-  $_waiting_for_event = 0;
 }
 
 # TODO - Ton Hospel's Tk event loop doesn't mix alarms and immediate
@@ -139,12 +132,7 @@ sub loop_do_timeslice {
   my $now;
   $now = time() if TRACE_STATISTICS;
 
-  # Idle until the next POE event is due.  Process at least one event
-  # to update the waiting flag, then block until a POE event is ready.
-  DoOneEvent(DONT_WAIT | ALL_EVENTS);
-  while ($_waiting_for_event) {
-    DoOneEvent(ALL_EVENTS);
-  }
+  DoOneEvent(ALL_EVENTS);
 
   $self->_data_stat_add('idle_seconds', time() - $now) if TRACE_STATISTICS;
 
@@ -171,7 +159,7 @@ __END__
 
 =head1 NAME
 
-POE::Loop::TkCommon - common features of POE's Tk event loop bridges
+POE::Loop::TkCommon - common code between the POE/Tk event loop bridges
 
 =head1 SYNOPSIS
 
@@ -179,13 +167,15 @@ See L<POE::Loop>.
 
 =head1 DESCRIPTION
 
-This class is an implementation of the abstract POE::Loop interface.
-It follows POE::Loop's public interface exactly.  Therefore, please
-see L<POE::Loop> for its documentation.
+POE::Loop::TkCommon is a mix-in class that supports common features
+between POE::Loop::Tk and POE::Loop::TkActiveState.  All Tk bridges
+implement the interface documented in POE::Loop.  Therefore, please
+see L<POE::Loop> for more details.
 
 =head1 SEE ALSO
 
-L<POE>, L<POE::Loop>, L<Tk>
+L<POE>, L<POE::Loop>, L<Tk>, L<POE::Loop::Tk>,
+L<POE::Loop::TkActiveState>
 
 =head1 AUTHORS & LICENSING
 
@@ -193,3 +183,5 @@ Please see L<POE> for more information about authors, contributors,
 and POE's licensing.
 
 =cut
+
+# rocco // vim: ts=2 sw=2 expandtab
