@@ -1,12 +1,12 @@
-# $Id: SIDs.pm 1903 2006-03-20 04:44:08Z rcaputo $
+# $Id: SIDs.pm 2335 2008-05-26 18:39:15Z rcaputo $
 
 # Session IDs: The data to maintain them, and accessors to get at them
 # sanely from other files.
 
-package POE::Resources::SIDs;
+package POE::Resource::SIDs;
 
 use vars qw($VERSION);
-$VERSION = do {my($r)=(q$Revision: 1903 $=~/(\d+)/);sprintf"1.%04d",$r};
+$VERSION = do {my($r)=(q$Revision: 2335 $=~/(\d+)/);sprintf"1.%04d",$r};
 
 # These methods are folded into POE::Kernel;
 package POE::Kernel;
@@ -32,7 +32,6 @@ sub _data_sid_initialize {
   $poe_kernel->[KR_SESSION_IDS] = \%kr_session_ids;
   $poe_kernel->[KR_SID_SEQ] = \$kr_sid_seq;
 }
-use POE::API::ResLoader \&_data_sid_initialize;
 
 ### End-run leak checking.
 
@@ -89,30 +88,37 @@ __END__
 
 =head1 NAME
 
-POE::Resources::SIDs - session ID management for POE::Kernel
+POE::Resource::SIDs - internal session ID manager for POE::Kernel
 
 =head1 SYNOPSIS
 
-Used internally by POE::Kernel.  Better documentation will be
-forthcoming.
+There is no public API.
 
 =head1 DESCRIPTION
 
-This module encapsulates and provides accessors for POE::Kernel's data
-structures that manage session IDs.  It is used internally by
-POE::Kernel and has no public interface.
+POE::Resource::SIDs is a mix-in class for POE::Kernel.  It provides
+the features necessary to manage session IDs.  It is used internally
+by POE::Kernel, so it has no public interface.
 
 =head1 SEE ALSO
 
-See L<POE::Kernel> and L<POE::Session> for documentation about session
-IDs.
+See L<POE::Kernel/Session Identifiers (IDs and Aliases)> for more
+information about session IDs.
+
+See L<POE::Kernel/Resources> for for public information about POE
+resources.
+
+See L<POE::Resource> for general discussion about resources and the
+classes that manage them.
 
 =head1 BUGS
 
-Probably.
+None known.
 
 =head1 AUTHORS & COPYRIGHTS
 
 Please see L<POE> for more information about authors and contributors.
 
 =cut
+
+# rocco // vim: ts=2 sw=2 expandtab
