@@ -1,11 +1,11 @@
-# $Id: Run.pm,v 1.70 2005/08/19 16:12:46 rcaputo Exp $
+# $Id: Run.pm 2358 2008-06-26 04:51:13Z rcaputo $
 
 package POE::Wheel::Run;
 
 use strict;
 
 use vars qw($VERSION);
-$VERSION = do {my@r=(q$Revision: 1.70 $=~/\d+/g);sprintf"%d."."%04d"x$#r,@r};
+$VERSION = do {my($r)=(q$Revision: 2358 $=~/(\d+)/);sprintf"1.%04d",$r};
 
 use Carp qw(carp croak);
 use POSIX qw(
@@ -36,33 +36,6 @@ BEGIN {
     else    { Win32API::File->import( qw(FdGetOsFHandle) ); };
   }
 
-  # How else can I get them out?!
-  if (eval '&IO::Tty::Constant::TIOCSCTTY') {
-    *TIOCSCTTY = *IO::Tty::Constant::TIOCSCTTY;
-  }
-  else {
-    eval 'sub TIOCSCTTY () { undef }';
-  }
-
-  if (eval '&IO::Tty::Constant::CIBAUD') {
-    *CIBAUD = *IO::Tty::Constant::CIBAUD;
-  }
-  else {
-    eval 'sub CIBAUD () { undef; }';
-  }
-
-  if (
-    eval '&IO::Tty::Constant::TIOCSWINSZ' and
-    eval '&IO::Tty::Constant::TIOCGWINSZ'
-  ) {
-    *TIOCSWINSZ = *IO::Tty::Constant::TIOCSWINSZ;
-    *TIOCGWINSZ = *IO::Tty::Constant::TIOCGWINSZ;
-  }
-  else {
-    eval 'sub TIOCSWINSZ () { undef; }';
-    eval 'sub TIOCGWINSZ () { undef; }';
-  }
-
   # Determine the most file descriptors we can use.
   my $max_open_fds;
   eval {
@@ -151,10 +124,11 @@ sub new {
     croak "$type\'s Conduit type ($conduit) is unknown" if (
       $conduit ne 'pipe' and
       $conduit ne 'pty'  and
+      $conduit ne 'pty-pipe' and
       $conduit ne 'socketpair' and
       $conduit ne 'inet'
     );
-    unless ($conduit eq "pty") {
+    unless ($conduit =~ /^pty(-pipe)?$/) {
       $stdio_type = $conduit;
       $conduit = "pipe";
     }
@@ -230,6 +204,7 @@ sub new {
   my $close_event = delete $params{CloseEvent};
 
   my $no_setsid = delete $params{NoSetSid};
+  my $no_setpgrp = delete $params{NoSetPgrp};
 
   # Make sure the user didn't pass in parameters we're not aware of.
   if (scalar keys %params) {
@@ -250,11 +225,17 @@ sub new {
   croak "could not create semaphore pipe: $!" unless defined $sem_pipe_read;
 
   # Use IO::Pty if requested.  IO::Pty turns on autoflush for us.
-  if ($conduit eq 'pty') {
+  if ($conduit =~ /^pty(-pipe)?$/) {
     croak "IO::Pty is not available" unless PTY_AVAILABLE;
 
     $stdin_write = $stdout_read = IO::Pty->new();
     croak "could not create master pty: $!" unless defined $stdout_read;
+
+    if ($conduit eq "pty-pipe") {
+      ($stderr_read, $stderr_write) = POE::Pipe::OneWay->new();
+      croak "could not make stderr pipes: $!"
+        unless defined $stderr_read and defined $stderr_write;
+    }
   }
 
   # Use pipes otherwise.
@@ -283,70 +264,48 @@ sub new {
   # Fork!  Woo-hoo!
   my $pid = fork;
 
-  # Stdio should not be tied.  Resolves rt.cpan.org ticket 1648.
-  if (tied *STDOUT) {
-    carp "Cannot redirect into tied STDOUT.  Untying it";
-    untie *STDOUT;
-  }
-  if (tied *STDERR) {
-    carp "Cannot redirect into tied STDERR.  Untying it";
-    untie *STDERR;
-  }
-
   # Child.  Parent side continues after this block.
   unless ($pid) {
     croak "couldn't fork: $!" unless defined $pid;
 
+    # Stdio should not be tied.  Resolves rt.cpan.org ticket 1648.
+    if (tied *STDOUT) {
+      carp "Cannot redirect into tied STDOUT.  Untying it";
+      untie *STDOUT;
+    }
+    if (tied *STDERR) {
+      carp "Cannot redirect into tied STDERR.  Untying it";
+      untie *STDERR;
+    }
+
     # If running pty, we delay the slave side creation 'til after
     # doing the necessary bits to become our own [unix] session.
-    if ($conduit eq 'pty') {
+    if ($conduit =~ /^pty(-pipe)?$/) {
 
       # Become a new unix session.
       # Program 19.3, APITUE.  W. Richard Stevens built my hot rod.
       eval 'setsid()' unless $no_setsid;
 
+      # Acquire a controlling terminal.  Program 19.3, APITUE.
+      $stdin_write->make_slave_controlling_terminal();
+
       # Open the slave side of the pty.
-      $stdin_read = $stdout_write = $stderr_write = $stdin_write->slave();
+      $stdin_read = $stdout_write = $stdin_write->slave();
       croak "could not create slave pty: $!" unless defined $stdin_read;
 
-      # Acquire a controlling terminal.  Program 19.3, APITUE.
-      if (defined TIOCSCTTY and not defined CIBAUD) {
-        ioctl( $stdin_read, TIOCSCTTY, 0 );
-      }
+      # For a simple pty conduit, stderr is wedged into stdout.
+      $stderr_write = $stdout_write if $conduit eq 'pty';
 
       # Put the pty conduit (slave side) into "raw" or "cbreak" mode,
       # per APITUE 19.4 and 11.10.
-      my $tio = POSIX::Termios->new();
-      $tio->getattr(fileno($stdin_read));
-      my $lflag = $tio->getlflag;
-      $lflag &= ~(ECHO | ICANON | IEXTEN | ISIG);
-      $tio->setlflag($lflag);
-      my $iflag = $tio->getiflag;
-      $iflag &= ~(BRKINT | ICRNL | INPCK | ISTRIP | IXON);
-      $tio->setiflag($iflag);
-      my $cflag = $tio->getcflag;
-      $cflag &= ~(CSIZE | PARENB);
-      $tio->setcflag($cflag);
-      my $oflag = $tio->getoflag;
-      $oflag &= ~(OPOST);
-      $tio->setoflag($oflag);
-      $tio->setattr(fileno($stdin_read), TCSANOW);
+      $stdin_read->set_raw();
 
       # Set the pty conduit (slave side) window size to our window
       # size.  APITUE 19.4 and 19.5.
-      if (defined TIOCGWINSZ) {
-        my $window_size = '!' x 25;
-        if (-t STDIN and !$winsize) {
-          ioctl( STDIN, TIOCGWINSZ, $window_size ) or die $!;
-        }
-        $window_size = pack('SSSS', @$winsize) if ref($winsize);
-        if ($window_size ne '!' x 25) {
-          ioctl( $stdin_read, TIOCSWINSZ, $window_size ) or die $!;
-        }
-        else {
-          carp "STDIN is not a terminal.  Can't set slave pty's window size";
-        }
-      }
+      eval { $stdin_read->clone_winsize_from(\*STDIN) };
+    }
+    else {
+      eval 'setpgrp(0,0)' unless $no_setpgrp;
     }
 
     # Reset all signals in the child process.  POE's own handlers are
@@ -355,7 +314,7 @@ sub new {
     my @safe_signals = $poe_kernel->_data_sig_get_safe_signals();
     @SIG{@safe_signals} = ("DEFAULT") x @safe_signals;
 
-    # -><- How to pass events to the parent process?  Maybe over a
+    # TODO How to pass events to the parent process?  Maybe over a
     # expedited (OOB) filehandle.
 
     # Fix the child process' priority.  Don't bother doing this if it
@@ -366,26 +325,26 @@ sub new {
       eval {
         if (defined(my $priority = getpriority(0, $$))) {
           unless (setpriority(0, $$, $priority + $priority_delta)) {
-            # -><- can't set child priority
+            # TODO can't set child priority
           }
         }
         else {
-          # -><- can't get child priority
+          # TODO can't get child priority
         }
       };
       if ($@) {
-        # -><- can't get child priority
+        # TODO can't get child priority
       }
     }
 
-    # Fix the group ID.  -><- Add getgrnam so group IDs can be
-    # specified by name.  -><- Warn if not superuser to begin with.
+    # Fix the group ID.  TODO Add getgrnam so group IDs can be
+    # specified by name.  TODO Warn if not superuser to begin with.
     if (defined $group_id) {
       $( = $) = $group_id;
     }
 
-    # Fix the user ID.  -><- Add getpwnam so user IDs can be specified
-    # by name.  -><- Warn if not superuser to begin with.
+    # Fix the user ID.  TODO Add getpwnam so user IDs can be specified
+    # by name.  TODO Warn if not superuser to begin with.
     if (defined $user_id) {
       $< = $> = $user_id;
     }
@@ -395,30 +354,26 @@ sub new {
     close $stdout_read;
     close $stderr_read if defined $stderr_read;
 
-    # Need to close on Win32 because std handles aren't dup'ed, no
-    # harm elsewhere.  Close STDERR later to not influence possible
-    # die.
-    close STDIN;
-    close STDOUT;
+    # Win32 needs the stdio handles closed before they're reopened
+    # because the standard handles aren't dup()'d.
 
     # Redirect STDIN from the read end of the stdin pipe.
+    close STDIN if POE::Kernel::RUNNING_IN_HELL;
     open( STDIN, "<&" . fileno($stdin_read) )
       or die "can't redirect STDIN in child pid $$: $!";
 
     # Redirect STDOUT to the write end of the stdout pipe.
     # The STDOUT_FILENO check snuck in on a patch.  I'm not sure why
     # we care what the file descriptor is.
+    close STDOUT if POE::Kernel::RUNNING_IN_HELL;
     open( STDOUT, ">&" . fileno($stdout_write) )
       or die "can't redirect stdout in child pid $$: $!";
 
-    # Need to close on Win32 because std handles aren't dup'ed, no
-    # harm elsewhere
-    close STDERR;
-
     # Redirect STDERR to the write end of the stderr pipe.  If the
     # stderr pipe's undef, then we use STDOUT.
     # The STDERR_FILENO check snuck in on a patch.  I'm not sure why
     # we care what the file descriptor is.
+    close STDERR if POE::Kernel::RUNNING_IN_HELL;
     open( STDERR, ">&" . fileno($stderr_write) )
       or die "can't redirect stderr in child: $!";
 
@@ -436,7 +391,7 @@ sub new {
       # based on the true win32 handles For the exec these get
       # remembered, so manipulation of STDIN/OUT/ERR is not enough.
       # Only necessary for the exec, as Perl CODE subroutine goes
-      # through 0/1/2 which are correct.  But ofcourse that coderef
+      # through 0/1/2 which are correct.  But of course that coderef
       # might invoke exec, so better do it regardless.
       # HACK: Using Win32::Console as nothing else exposes SetStdHandle
       Win32::Console::_SetStdHandle(
@@ -475,14 +430,16 @@ sub new {
 
       # Try to exit without triggering END or object destructors.
       # Give up with a plain exit if we must.
-      # On win32 cannot _exit as it will kill *all* threads, meaning parent too
+      # But we can't _exit on Win32 because it KILLS ALL THREADS,
+      # including the parent "process".
       unless (POE::Kernel::RUNNING_IN_HELL) {
         eval { POSIX::_exit(0);  };
         eval { kill KILL => $$;  };
         eval { exec("$^X -e 0"); };
       };
       exit(0);
-    } else {
+    }
+    else {
       if (ref($program) eq 'ARRAY') {
         exec(@$program, @$prog_args)
           or die "can't exec (@$program) in child pid $$: $!";
@@ -500,9 +457,9 @@ sub new {
   close $stdout_write if defined $stdout_write;
   close $stderr_write if defined $stderr_write;
 
-  my $handle_count = 0;
-  $handle_count++ if defined $stdout_read;
-  $handle_count++ if defined $stderr_read;
+  my $active_count = 0;
+  $active_count++ if $stdout_event and $stdout_read;
+  $active_count++ if $stderr_event and $stderr_read;
 
   my $self = bless [
     &POE::Wheel::allocate_wheel_id(),  # UNIQUE_ID
@@ -511,7 +468,7 @@ sub new {
     $program,       # PROGRAM
     $pid,           # CHILD_PID
     $conduit,       # CONDUIT_TYPE
-    $handle_count,  # IS_ACTIVE
+    $active_count,  # IS_ACTIVE
     $close_on_call, # CLOSE_ON_CALL
     $stdio_type,    # STDIO_TYPE
     # STDIN
@@ -536,7 +493,10 @@ sub new {
   ], $type;
 
   # Wait here while the child sets itself up.
-  <$sem_pipe_read>;
+  {
+    local $/ = "\n";
+    <$sem_pipe_read>;
+  }
   close $sem_pipe_read;
   close $sem_pipe_write;
 
@@ -617,96 +577,99 @@ sub _define_stdin_flusher {
 sub _define_stdout_reader {
   my $self = shift;
 
+  # Can't do anything if we don't have a handle.
+  return unless defined $self->[HANDLE_STDOUT];
+
+  # No event?  Unregister the handler and leave.
+  my $stdout_event  = \$self->[EVENT_STDOUT];
+  unless ($$stdout_event) {
+    $poe_kernel->select_read($self->[HANDLE_STDOUT]);
+    return;
+  }
+
+  # If any of these change, then the read state is invalidated and
+  # needs to be redefined.
+  my $unique_id     = $self->[UNIQUE_ID];
+  my $driver        = $self->[DRIVER_STDOUT];
+  my $stdout_filter = $self->[FILTER_STDOUT];
+
+  # These can change without redefining the callback since they're
+  # enclosed by reference.
+  my $is_active     = \$self->[IS_ACTIVE];
+  my $close_event   = \$self->[CLOSE_EVENT];
+  my $error_event   = \$self->[ERROR_EVENT];
+
   # Register the select-read handler for STDOUT.
-  if (defined $self->[HANDLE_STDOUT]) {
+  if (
+    $stdout_filter->can("get_one") and
+    $stdout_filter->can("get_one_start")
+  ) {
+    $poe_kernel->state(
+      $self->[STATE_STDOUT] = ref($self) . "($unique_id) -> select stdout",
+      sub {
+        # prevents SEGV
+        0 && CRIMSON_SCOPE_HACK('<');
 
-    # If any of these change, then the read state is invalidated and
-    # needs to be redefined.
-    my $unique_id     = $self->[UNIQUE_ID];
-    my $driver        = $self->[DRIVER_STDOUT];
-    my $error_event   = \$self->[ERROR_EVENT];
-    my $close_event   = \$self->[CLOSE_EVENT];
-    my $stdout_filter = $self->[FILTER_STDOUT];
-    my $stdout_event  = \$self->[EVENT_STDOUT];
-    my $is_active     = \$self->[IS_ACTIVE];
-
-    if (
-      $stdout_filter->can("get_one") and
-      $stdout_filter->can("get_one_start")
-    ) {
-      $poe_kernel->state(
-        $self->[STATE_STDOUT] = ref($self) . "($unique_id) -> select stdout",
-        sub {
-          # prevents SEGV
-          0 && CRIMSON_SCOPE_HACK('<');
-
-          # subroutine starts here
-          my ($k, $me, $handle) = @_[KERNEL, SESSION, ARG0];
-          if (defined(my $raw_input = $driver->get($handle))) {
-            $stdout_filter->get_one_start($raw_input);
-            while (1) {
-              my $next_rec = $stdout_filter->get_one();
-              last unless @$next_rec;
-              foreach my $cooked_input (@$next_rec) {
-                $k->call($me, $$stdout_event, $cooked_input, $unique_id);
-              }
-            }
-          }
-          else {
-            $$error_event and $k->call(
-              $me, $$error_event,
-              'read', ($!+0), $!, $unique_id, 'STDOUT'
-            );
-            unless (--$$is_active) {
-              $k->call( $me, $$close_event, $unique_id )
-                if defined $$close_event;
-            }
-            $k->select_read($handle);
-          }
-        }
-      );
-    }
-
-    # Otherwise we can't get one.
-    else {
-      $poe_kernel->state(
-        $self->[STATE_STDOUT] = ref($self) . "($unique_id) -> select stdout",
-        sub {
-          # prevents SEGV
-          0 && CRIMSON_SCOPE_HACK('<');
-
-          # subroutine starts here
-          my ($k, $me, $handle) = @_[KERNEL, SESSION, ARG0];
-          if (defined(my $raw_input = $driver->get($handle))) {
-            foreach my $cooked_input (@{$stdout_filter->get($raw_input)}) {
+        # subroutine starts here
+        my ($k, $me, $handle) = @_[KERNEL, SESSION, ARG0];
+        if (defined(my $raw_input = $driver->get($handle))) {
+          $stdout_filter->get_one_start($raw_input);
+          while (1) {
+            my $next_rec = $stdout_filter->get_one();
+            last unless @$next_rec;
+            foreach my $cooked_input (@$next_rec) {
               $k->call($me, $$stdout_event, $cooked_input, $unique_id);
             }
           }
-          else {
-            $$error_event and
-              $k->call(
-                $me, $$error_event,
-                'read', ($!+0), $!, $unique_id, 'STDOUT'
-              );
-            unless (--$$is_active) {
-              $k->call( $me, $$close_event, $unique_id )
-                if defined $$close_event;
-            }
-            $k->select_read($handle);
+        }
+        else {
+          $$error_event and $k->call(
+            $me, $$error_event,
+            'read', ($!+0), $!, $unique_id, 'STDOUT'
+          );
+          unless (--$$is_active) {
+            $k->call( $me, $$close_event, $unique_id )
+              if defined $$close_event;
+          }
+          $k->select_read($handle);
+        }
+      }
+    );
+  }
+
+  # Otherwise we can't get one.
+  else {
+    $poe_kernel->state(
+      $self->[STATE_STDOUT] = ref($self) . "($unique_id) -> select stdout",
+      sub {
+        # prevents SEGV
+        0 && CRIMSON_SCOPE_HACK('<');
+
+        # subroutine starts here
+        my ($k, $me, $handle) = @_[KERNEL, SESSION, ARG0];
+        if (defined(my $raw_input = $driver->get($handle))) {
+          foreach my $cooked_input (@{$stdout_filter->get($raw_input)}) {
+            $k->call($me, $$stdout_event, $cooked_input, $unique_id);
           }
         }
-      );
-    }
-
-    # register the state's select
-    $poe_kernel->select_read($self->[HANDLE_STDOUT], $self->[STATE_STDOUT]);
+        else {
+          $$error_event and
+            $k->call(
+              $me, $$error_event,
+              'read', ($!+0), $!, $unique_id, 'STDOUT'
+            );
+          unless (--$$is_active) {
+            $k->call( $me, $$close_event, $unique_id )
+              if defined $$close_event;
+          }
+          $k->select_read($handle);
+        }
+      }
+    );
   }
 
-  # Register the select-read handler for STDOUT.
-  else {
-    $poe_kernel->select_read($self->[HANDLE_STDOUT])
-      if defined $self->[HANDLE_STDOUT];
-  }
+  # register the state's select
+  $poe_kernel->select_read($self->[HANDLE_STDOUT], $self->[STATE_STDOUT]);
 }
 
 #------------------------------------------------------------------------------
@@ -716,94 +679,96 @@ sub _define_stdout_reader {
 sub _define_stderr_reader {
   my $self = shift;
 
+  # Can't do anything if we don't have a handle.
+  return unless defined $self->[HANDLE_STDERR];
+
+  # No event?  Unregister the handler and leave.
+  my $stderr_event  = \$self->[EVENT_STDERR];
+  unless ($$stderr_event) {
+    $poe_kernel->select_read($self->[HANDLE_STDERR]);
+    return;
+  }
+
+  my $unique_id     = $self->[UNIQUE_ID];
+  my $driver        = $self->[DRIVER_STDERR];
+  my $stderr_filter = $self->[FILTER_STDERR];
+
+  # These can change without redefining the callback since they're
+  # enclosed by reference.
+  my $error_event   = \$self->[ERROR_EVENT];
+  my $close_event   = \$self->[CLOSE_EVENT];
+  my $is_active     = \$self->[IS_ACTIVE];
+
   # Register the select-read handler for STDERR.
-  if (defined $self->[HANDLE_STDERR]) {
-    # If any of these change, then the read state is invalidated and
-    # needs to be redefined.
-    my $unique_id     = $self->[UNIQUE_ID];
-    my $driver        = $self->[DRIVER_STDERR];
-    my $error_event   = \$self->[ERROR_EVENT];
-    my $close_event   = \$self->[CLOSE_EVENT];
-    my $stderr_filter = $self->[FILTER_STDERR];
-    my $stderr_event  = \$self->[EVENT_STDERR];
-    my $is_active     = \$self->[IS_ACTIVE];
+  if (
+    $stderr_filter->can("get_one") and
+    $stderr_filter->can("get_one_start")
+  ) {
+    $poe_kernel->state(
+      $self->[STATE_STDERR] = ref($self) . "($unique_id) -> select stderr",
+      sub {
+        # prevents SEGV
+        0 && CRIMSON_SCOPE_HACK('<');
 
-    if (
-      $stderr_filter->can("get_one") and
-      $stderr_filter->can("get_one_start")
-    ) {
-      $poe_kernel->state(
-        $self->[STATE_STDERR] = ref($self) . "($unique_id) -> select stderr",
-        sub {
-          # prevents SEGV
-          0 && CRIMSON_SCOPE_HACK('<');
-
-          # subroutine starts here
-          my ($k, $me, $handle) = @_[KERNEL, SESSION, ARG0];
-          if (defined(my $raw_input = $driver->get($handle))) {
-            $stderr_filter->get_one_start($raw_input);
-            while (1) {
-              my $next_rec = $stderr_filter->get_one();
-              last unless @$next_rec;
-              foreach my $cooked_input (@$next_rec) {
-                $k->call($me, $$stderr_event, $cooked_input, $unique_id);
-              }
-            }
-          }
-          else {
-            $$error_event and $k->call(
-              $me, $$error_event,
-              'read', ($!+0), $!, $unique_id, 'STDERR'
-            );
-            unless (--$$is_active) {
-              $k->call( $me, $$close_event, $unique_id )
-                if defined $$close_event;
-            }
-            $k->select_read($handle);
-          }
-        }
-      );
-    }
-
-    # Otherwise we can't get_one().
-    else {
-      $poe_kernel->state(
-        $self->[STATE_STDERR] = ref($self) . "($unique_id) -> select stderr",
-        sub {
-          # prevents SEGV
-          0 && CRIMSON_SCOPE_HACK('<');
-
-          # subroutine starts here
-          my ($k, $me, $handle) = @_[KERNEL, SESSION, ARG0];
-          if (defined(my $raw_input = $driver->get($handle))) {
-            foreach my $cooked_input (@{$stderr_filter->get($raw_input)}) {
+        # subroutine starts here
+        my ($k, $me, $handle) = @_[KERNEL, SESSION, ARG0];
+        if (defined(my $raw_input = $driver->get($handle))) {
+          $stderr_filter->get_one_start($raw_input);
+          while (1) {
+            my $next_rec = $stderr_filter->get_one();
+            last unless @$next_rec;
+            foreach my $cooked_input (@$next_rec) {
               $k->call($me, $$stderr_event, $cooked_input, $unique_id);
             }
           }
-          else {
-            $$error_event and $k->call(
-              $me, $$error_event,
-              'read', ($!+0), $!, $unique_id, 'STDERR'
-            );
-            unless (--$$is_active) {
-              $k->call( $me, $$close_event, $unique_id )
-                if defined $$close_event;
-            }
-            $k->select_read($handle);
+        }
+        else {
+          $$error_event and $k->call(
+            $me, $$error_event,
+            'read', ($!+0), $!, $unique_id, 'STDERR'
+          );
+          unless (--$$is_active) {
+            $k->call( $me, $$close_event, $unique_id )
+              if defined $$close_event;
+          }
+          $k->select_read($handle);
+        }
+      }
+    );
+  }
+
+  # Otherwise we can't get_one().
+  else {
+    $poe_kernel->state(
+      $self->[STATE_STDERR] = ref($self) . "($unique_id) -> select stderr",
+      sub {
+        # prevents SEGV
+        0 && CRIMSON_SCOPE_HACK('<');
+
+        # subroutine starts here
+        my ($k, $me, $handle) = @_[KERNEL, SESSION, ARG0];
+        if (defined(my $raw_input = $driver->get($handle))) {
+          foreach my $cooked_input (@{$stderr_filter->get($raw_input)}) {
+            $k->call($me, $$stderr_event, $cooked_input, $unique_id);
           }
         }
-      );
-    }
-
-    # register the state's select
-    $poe_kernel->select_read($self->[HANDLE_STDERR], $self->[STATE_STDERR]);
+        else {
+          $$error_event and $k->call(
+            $me, $$error_event,
+            'read', ($!+0), $!, $unique_id, 'STDERR'
+          );
+          unless (--$$is_active) {
+            $k->call( $me, $$close_event, $unique_id )
+              if defined $$close_event;
+          }
+          $k->select_read($handle);
+        }
+      }
+    );
   }
 
-  # Register the select-read handler for STDERR.
-  else {
-    $poe_kernel->select_read($self->[HANDLE_STDERR])
-      if defined $self->[HANDLE_STDERR];
-  }
+  # Register the state's select.
+  $poe_kernel->select_read($self->[HANDLE_STDERR], $self->[STATE_STDERR]);
 }
 
 #------------------------------------------------------------------------------
@@ -846,9 +811,11 @@ sub event {
     }
   }
 
-  $self->_define_stdin_flusher() if defined $redefine_stdin;
-  $self->_define_stdout_reader() if defined $redefine_stdout;
-  $self->_define_stderr_reader() if defined $redefine_stderr;
+  # Recalculate the active handles count.
+  my $active_count = 0;
+  $active_count++ if $self->[EVENT_STDOUT] and $self->[HANDLE_STDOUT];
+  $active_count++ if $self->[EVENT_STDERR] and $self->[HANDLE_STDERR];
+  $self->[IS_ACTIVE] = $active_count;
 }
 
 #------------------------------------------------------------------------------
@@ -893,6 +860,12 @@ sub DESTROY {
 
 sub put {
   my ($self, @chunks) = @_;
+
+  # Avoid big bada boom if someone put()s on a dead wheel.
+  croak "Called put() on a wheel without an open STDIN handle" unless (
+    $self->[HANDLE_STDIN]
+  );
+
   if (
     $self->[OCTETS_STDIN] =  # assignment on purpose
     $self->[DRIVER_STDIN]->put($self->[FILTER_STDIN]->put(\@chunks))
@@ -939,7 +912,11 @@ sub shutdown_stdin {
   $poe_kernel->select_write($self->[HANDLE_STDIN], undef);
 
   eval { local $^W = 0; shutdown($self->[HANDLE_STDIN], 1) };
-  close $self->[HANDLE_STDIN] if $@;
+  if ($@ or $self->[HANDLE_STDIN] != $self->[HANDLE_STDOUT]) {
+    close $self->[HANDLE_STDIN];
+  }
+
+  $self->[HANDLE_STDIN] = undef;
 }
 
 #------------------------------------------------------------------------------
@@ -1049,7 +1026,6 @@ sub set_stdout_filter {
   my $buf = $self->[FILTER_STDOUT]->get_pending();
   $self->[FILTER_STDOUT] = $new_filter;
 
-  $self->_define_stdout_reader();
   $self->_transfer_stdout_buffer($buf);
 }
 
@@ -1059,7 +1035,6 @@ sub set_stderr_filter {
   my $buf = $self->[FILTER_STDERR]->get_pending();
   $self->[FILTER_STDERR] = $new_filter;
 
-  $self->_define_stderr_reader();
   $self->_transfer_stderr_buffer($buf);
 }
 
@@ -1103,14 +1078,13 @@ sub kill {
   eval { kill $signal, $self->[CHILD_PID] };
 }
 
-###############################################################################
 1;
 
 __END__
 
 =head1 NAME
 
-POE::Wheel::Run - event driven fork/exec with added value
+POE::Wheel::Run - abstract pipe/fork/exec mix-in (also runs subroutines)
 
 =head1 SYNOPSIS
 
@@ -1119,35 +1093,33 @@ POE::Wheel::Run - event driven fork/exec with added value
   $program = [ '/usr/bin/cat', '-' ];
 
   $wheel = POE::Wheel::Run->new(
+    # Set the program to execute, and optionally some parameters.
     Program     => $program,
-    ProgramArgs => \@program_args,     # Parameters for $program.
-    Priority    => +5,                 # Adjust priority.  May need to be root.
-    User        => getpwnam('nobody'), # Adjust UID. May need to be root.
-    Group       => getgrnam('nobody'), # Adjust GID. May need to be root.
-    ErrorEvent  => 'oops',             # Event to emit on errors.
-    CloseEvent  => 'child_closed',     # Child closed all output.
+    ProgramArgs => \@program_args,
 
-    StdinEvent  => 'stdin',  # Event to emit when stdin is flushed to child.
-    StdoutEvent => 'stdout', # Event to emit with child stdout information.
-    StderrEvent => 'stderr', # Event to emit with child stderr information.
+    # Define I/O events to emit.  Most are optional.
+    StdinEvent  => 'stdin',     # Flushed all data to the child's STDIN.
+    StdoutEvent => 'stdout',    # Received data from the child's STDOUT.
+    StderrEvent => 'stderr',    # Received data from the child's STDERR.
+    ErrorEvent  => 'oops',          # An I/O error occurred.
+    CloseEvent  => 'child_closed',  # Child closed all output handles.
 
-    # Specify different I/O formats.
+    # Optionally adjust the child process priority, user ID, and/or
+    # group ID.  You may need to be root to do this.
+    Priority    => +5,
+    User        => scalar(getpwnam 'nobody'),
+    Group       => scalar(getgrnam 'nobody'),
+
+    # Optionally specify different I/O formats.
     StdinFilter  => POE::Filter::Line->new(),   # Child accepts input as lines.
     StdoutFilter => POE::Filter::Stream->new(), # Child output is a stream.
     StderrFilter => POE::Filter::Line->new(),   # Child errors are lines.
 
-    # Set StdinFilter and StdoutFilter together.
+    # Shorthand to set StdinFilter and StdoutFilter together.
     StdioFilter => POE::Filter::Line->new(),    # Or some other filter.
-
-    # Specify different I/O methods.
-    StdinDriver  => POE::Driver::SysRW->new(),  # Defaults to SysRW.
-    StdoutDriver => POE::Driver::SysRW->new(),  # Same.
-    StderrDriver => POE::Driver::SysRW->new(),  # Same.
-
-    # Set StdinDriver and StdoutDriver together.
-    StdioDriver  => POE::Driver::SysRW->new(),
   );
 
+  # Information about the wheel and its process.
   print "Unique wheel ID is  : ", $wheel->ID;
   print "Wheel's child PID is: ", $wheel->PID;
 
@@ -1155,8 +1127,7 @@ POE::Wheel::Run - event driven fork/exec with added value
   $wheel->put( 'input for the child' );
 
   # Kill the child.
-  $wheel->kill();  # TERM by default
-  $wheel->kill(9);
+  $wheel->kill(9);  # TERM by default.
 
 =head1 DESCRIPTION
 
@@ -1198,7 +1169,9 @@ process.  By default it will try various forms of inter-process
 communication to build a pipe between the parent and child processes.
 If a particular method is preferred, it can be set to "pipe",
 "socketpair", or "inet".  It may also be set to "pty" if the child
-process should have its own pseudo tty.
+process should have its own pseudo tty.  Setting it to "pty-pipe"
+gives the child process a stdin and stdout pseudo-tty, but keeps
+stderr as a pipe, rather than merging stdout and stderr as with "pty".
 
 The reasons to define this parameter would be if you want to use
 "pty", if the default pipe type doesn't work properly on your
@@ -1256,6 +1229,9 @@ same time.
 
 =item StderrEvent
 
+See L<EVENTS AND PARAMETERS> below for a more detailed description of
+these events and their parameters.
+
 C<CloseEvent> contains the name of an event to emit when the child
 process closes all its output handles.  This is a consistent
 notification that the child will not be sending any more output.  It
@@ -1264,17 +1240,13 @@ accepting input.
 
 C<ErrorEvent> contains the name of an event to emit if something
 fails.  It is optional and if omitted, the wheel will not notify its
-session if any errors occur.  The event receives 5 parameters as
-follows: ARG0 = the return value of syscall(), ARG1 = errno() - the
-numeric value of the error generated, ARG2 = error() - a descriptive
-for the given error, ARG3 = the wheel id, and ARG4 = the handle on
-which the error occurred (stdout, stderr, etc.)
+session if any errors occur.
 
 Wheel::Run requires at least one of the following three events:
 
 C<StdinEvent> contains the name of an event that Wheel::Run emits
-whenever all its output has been flushed to the child process' STDIN
-handle.
+whenever everything queued by its put() method has been flushed to the
+child's STDIN handle.
 
 C<StdoutEvent> and C<StderrEvent> contain names of events that
 Wheel::Run emits whenever the child process writes something to its
@@ -1295,7 +1267,7 @@ methods.  If stderr is also to be used, StderrFilter will need to be
 specified separately.
 
 C<Filter> is optional.  If left blank, it will default to an
-instance of C<POE::Filter::Line->new(Literal => "\n");>
+instance of C<< POE::Filter::Line->new(Literal => "\n"); >>
 
 C<StdinFilter> and C<StdoutFilter> can be used instead of or in
 addition to C<StdioFilter>.  They will override the default filter's
@@ -1315,6 +1287,13 @@ When true, C<NoSetSid> disables setsid() in the child process.  By
 default, setsid() is called to execute the child process in a separate
 Unix session.
 
+=item NoSetPgrp
+
+When true, C<NoSetPgrp> disables setprgp() in the child process. By 
+default, setprgp() is called to change the process group for the child 
+process, if the OS supports process groups. If the conduit is a pty or
+pty-pipe setsid() is used instead, see C<NoSetSid>.
+
 =item Priority
 
 C<Priority> contains an offset from the current process's priority.
@@ -1338,7 +1317,9 @@ If C<Program> holds a code reference, it will be called in the forked
 child process, and then the child will exit.  This allows Wheel::Run
 to fork off bits of long-running code which can accept STDIN input and
 pass responses to STDOUT and/or STDERR.  Note, however, that POE's
-services are effectively disabled in the child process.
+services are effectively disabled in the child process. See
+L</Nested POE Kernel> for instructions on how to properly use POE within the
+child.
 
 L<perlfunc> has more information about exec() and the different ways
 to call it.
@@ -1433,6 +1414,12 @@ tend to be reluctant to exit when their terminals are closed.
 
 The kill() method will send SIGTERM if SIGNAL is undef or omitted.
 
+=item get_driver_out_messages
+
+=item get_driver_out_octets
+
+Return driver statistics.
+
 =back
 
 =head1 EVENTS AND PARAMETERS
@@ -1466,11 +1453,13 @@ whenever an error occurs.  Every error event comes with four
 parameters:
 
 C<ARG0> contains the name of the operation that failed.  It may be
-'read' or 'write' or 'fork' or 'exec' or something.  The actual values
-aren't yet defined.  Note: This is not necessarily a function name.
+'read', 'write', 'fork', 'exec' or the name of some other function or
+task.  The actual values aren't yet defined.  Note: This is not
+necessarily a function name.
 
 C<ARG1> and C<ARG2> hold numeric and string values for C<$!>,
-respectively.
+respectively.  C<"$!"> will eq C<""> for read error 0 (child process
+closed STDOUT or STDERR).
 
 C<ARG3> contains the wheel's unique ID.
 
@@ -1484,6 +1473,7 @@ A sample error event handler:
 
   sub error_state {
     my ($operation, $errnum, $errstr, $wheel_id) = @_[ARG0..ARG3];
+    $errstr = "remote end closed" if $operation eq "read" and !$errnum;
     warn "Wheel $wheel_id generated $operation error $errnum: $errstr\n";
   }
 
@@ -1500,9 +1490,9 @@ StdinEvent's C<ARG0> parameter contains its wheel's unique ID.
 =item StderrEvent
 
 StdoutEvent and StderrEvent contain names for events that Wheel::Run
-emits whenever the child process makes output.  StdoutEvent contains
-information the child wrote to its STDOUT handle, and StderrEvent
-includes whatever arrived from the child's STDERR handle.
+emits whenever the child process generates new output.  StdoutEvent
+contains information the child wrote to its STDOUT handle, and
+StderrEvent includes whatever arrived from the child's STDERR handle.
 
 Both of these events come with two parameters.  C<ARG0> contains the
 information that the child wrote.  C<ARG1> holds the wheel's unique
@@ -1522,6 +1512,8 @@ ID.
 
 =head1 TIPS AND TRICKS
 
+=head2 Execution environment
+
 One common task is scrubbing a child process' environment.  This
 amounts to clearing the contents of %ENV and setting it up with some
 known, secure values.
@@ -1544,6 +1536,42 @@ that performs the exec() call for us.
 That deletes everything from the environment, sets a simple, secure
 PATH, and executes a program with its arguments.
 
+=head2 Nested POE Kernel
+
+When the process forks the kernel's queue is effectively duplicated, giving the
+child process it's own copy.
+
+This means that if you call C<< $poe_kernel->run >> in the coderef variant of
+C<Program>, the kernel will deliver all currently queued events twice, causing
+the universe to implode, or worse.
+
+This is why L<POE::Wheel::Run> will exit immediately after executing the code
+reference, before the kernel's loop can dequeue anything more.
+
+In order to allow a POE kernel to be run inside the child process without
+having to C<exec> a new perl script with a fresh L<POE> environment, you can
+call the L<POE::Kernel/stop> method.
+
+Here is an example:
+
+  Program => sub {
+    $poe_kernel->stop; # flush all the kernel structures
+
+    # now that the kernel is clean we can do POEish stuff:
+    POE::Session->create(
+      ...
+    );
+
+    $poe_kernel->run; # this DWIMs now
+  }
+
+If you do not call L<POE::Kernel/stop>, but do call L<POE::Kernel/run> inside
+the child process strange things are bound to happen.
+
+The advantage of calling C<POE::Kernel/stop> is that it allows all the
+advantages of a C<fork> without an C<exec>, namely sharing of read only data,
+but without having to forefit L<POE>'s facilities in the child.
+
 =head1 SEE ALSO
 
 POE::Wheel.
@@ -1573,3 +1601,6 @@ names.
 Please see L<POE> for more information about authors and contributors.
 
 =cut
+
+# rocco // vim: ts=2 sw=2 expandtab
+# TODO - Redocument.
